/* Java Grid interface code.
   Copyright (C) 2001-2010 Roberto Bagnara <bagnara@cs.unipr.it>
   Copyright (C) 2010-2011 BUGSENG srl (http://bugseng.com)

This file is part of the Parma Polyhedra Library (PPL).

The PPL is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

The PPL is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1307, USA.

For the most up-to-date information see the Parma Polyhedra Library
site: http://www.cs.unipr.it/ppl/ . */

#include "ppl_java_common.defs.hh"
#include "parma_polyhedra_library_Grid.h"
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::Interfaces::Java;

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_finalize
(JNIEnv* env, jobject j_this) try {
  if (!is_java_marked(env, j_this)) {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    delete this_ptr;
  }
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__JLparma_1polyhedra_1library_Degenerate_1Element_2
(JNIEnv* env, jobject j_this, jlong j_dim, jobject j_degenerate_element) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(j_dim);
  jint j_degenerate_element_int
    = env->CallIntMethod(j_degenerate_element,
                         cached_FMIDs.Degenerate_Element_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_degenerate_element_int) {
  case 0:
    this_ptr = new Grid(ppl_dim, UNIVERSE);
    break;
  case 1:
    this_ptr = new Grid(ppl_dim, EMPTY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_C_1Polyhedron_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  C_Polyhedron* y_ptr
    = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_NNC_1Polyhedron_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  NNC_Polyhedron* y_ptr
    = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Grid_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Rational_1Box_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Rational_Box* y_ptr
    = reinterpret_cast<Rational_Box*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpz_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  BD_Shape<mpz_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpz_class>*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpq_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  BD_Shape<mpq_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpq_class>*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpz_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Octagonal_Shape<mpz_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpz_class>*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpq_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Octagonal_Shape<mpq_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpq_class>*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Double_1Box_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Double_Box* y_ptr
    = reinterpret_cast<Double_Box*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1double_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  BD_Shape<double>* y_ptr
    = reinterpret_cast<BD_Shape<double>*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1double_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Octagonal_Shape<double>* y_ptr
    = reinterpret_cast<Octagonal_Shape<double>*>(get_ptr(env, j_y));
  Grid* this_ptr = new Grid(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL







JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_C_1Polyhedron_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  C_Polyhedron* y_ptr
    = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_NNC_1Polyhedron_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  NNC_Polyhedron* y_ptr
    = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Grid_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Rational_1Box_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Rational_Box* y_ptr
    = reinterpret_cast<Rational_Box*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpz_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  BD_Shape<mpz_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpz_class>*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpq_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  BD_Shape<mpq_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpq_class>*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpz_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Octagonal_Shape<mpz_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpz_class>*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpq_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Octagonal_Shape<mpq_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpq_class>*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Double_1Box_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Double_Box* y_ptr
    = reinterpret_cast<Double_Box*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1double_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  BD_Shape<double>* y_ptr
    = reinterpret_cast<BD_Shape<double>*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1double_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Octagonal_Shape<double>* y_ptr
    = reinterpret_cast<Octagonal_Shape<double>*>(get_ptr(env, j_y));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  Grid* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new Grid(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new Grid(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new Grid(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL







JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Constraint_1System_2
(JNIEnv* env, jobject j_this, jobject j_iterable) try {
  Constraint_System cs
    = build_cxx_constraint_system(env, j_iterable);
  Grid* this_ptr = new Grid(cs, Recycle_Input());
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Congruence_1System_2
(JNIEnv* env, jobject j_this, jobject j_iterable) try {
  Congruence_System cs
    = build_cxx_congruence_system(env, j_iterable);
  Grid* this_ptr = new Grid(cs, Recycle_Input());
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_build_1cpp_1object__Lparma_1polyhedra_1library_Grid_1Generator_1System_2
(JNIEnv* env, jobject j_this, jobject j_iterable) try {
  Grid_Generator_System cs
    = build_cxx_grid_generator_system(env, j_iterable);
  Grid* this_ptr = new Grid(cs, Recycle_Input());
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL








JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_swap
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  this_ptr->swap(*y_ptr);
}
CATCH_ALL





JNIEXPORT jlong JNICALL
Java_parma_1polyhedra_1library_Grid_space_1dimension
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->space_dimension();
  }
  CATCH_ALL;
  return 0;
}

JNIEXPORT jlong JNICALL
Java_parma_1polyhedra_1library_Grid_affine_1dimension
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->affine_dimension();
  }
  CATCH_ALL;
  return 0;
}






JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_relation_1with__Lparma_1polyhedra_1library_Constraint_2
(JNIEnv* env, jobject j_this, jobject j_c) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Constraint c = build_cxx_constraint(env, j_c);
    Poly_Con_Relation pcr = this_ptr->relation_with(c);
    return build_java_poly_con_relation(env, pcr);
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_relation_1with__Lparma_1polyhedra_1library_Generator_2
(JNIEnv* env, jobject j_this, jobject j_c) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Generator c = build_cxx_generator(env, j_c);
    Poly_Gen_Relation pcr = this_ptr->relation_with(c);
    return build_java_poly_gen_relation(env, pcr);
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_relation_1with__Lparma_1polyhedra_1library_Congruence_2
(JNIEnv* env, jobject j_this, jobject j_c) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Congruence c = build_cxx_congruence(env, j_c);
    Poly_Con_Relation pcr = this_ptr->relation_with(c);
    return build_java_poly_con_relation(env, pcr);
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_relation_1with__Lparma_1polyhedra_1library_Grid_1Generator_2
(JNIEnv* env, jobject j_this, jobject j_c) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Grid_Generator c = build_cxx_grid_generator(env, j_c);
    Poly_Gen_Relation pcr = this_ptr->relation_with(c);
    return build_java_poly_gen_relation(env, pcr);
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}






JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_constraints
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return build_java_constraint_system(env, this_ptr->constraints());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_congruences
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return build_java_congruence_system(env, this_ptr->congruences());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_grid_1generators
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return build_java_grid_generator_system(env, this_ptr->grid_generators());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}






JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_minimized_1constraints
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return build_java_constraint_system(env,
             this_ptr->minimized_constraints());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_minimized_1congruences
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return build_java_congruence_system(env,
             this_ptr->minimized_congruences());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Grid_minimized_1grid_1generators
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return build_java_grid_generator_system(env,
             this_ptr->minimized_grid_generators());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_is_1empty
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->is_empty();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_is_1universe
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->is_universe();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_is_1bounded
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->is_bounded();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_contains_1integer_1point
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->contains_integer_point();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_is_1topologically_1closed
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->is_topologically_closed();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_is_1discrete
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->is_discrete();
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_topological_1closure_1assign
(JNIEnv* env, jobject j_this) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  this_ptr->topological_closure_assign();
}
CATCH_ALL






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_bounds_1from_1above
(JNIEnv* env, jobject j_this, jobject j_le) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    return this_ptr->bounds_from_above(le);
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_bounds_1from_1below
(JNIEnv* env, jobject j_this, jobject j_le) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    return this_ptr->bounds_from_below(le);
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_maximize__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_By_1Reference_2
(JNIEnv* env, jobject j_this, jobject j_le,
 jobject j_num, jobject j_den, jobject j_ref_boolean) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    PPL_DIRTY_TEMP_COEFFICIENT(num);
    PPL_DIRTY_TEMP_COEFFICIENT(den);
    num = build_cxx_coeff(env, j_num);
    den = build_cxx_coeff(env, j_den);
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    bool b_value;
    if (this_ptr->maximize(le, num, den, b_value)) {
      set_coefficient(env, j_num, build_java_coeff(env, num));
      set_coefficient(env, j_den, build_java_coeff(env, den));
      jobject j_boolean = bool_to_j_boolean_class(env, b_value);
      set_by_reference(env, j_ref_boolean, j_boolean);
      return true;
    }
    return false;
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_minimize__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_By_1Reference_2
(JNIEnv* env, jobject j_this, jobject j_le,
 jobject j_num, jobject j_den, jobject j_ref_boolean) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    PPL_DIRTY_TEMP_COEFFICIENT(num);
    PPL_DIRTY_TEMP_COEFFICIENT(den);
    num = build_cxx_coeff(env, j_num);
    den = build_cxx_coeff(env, j_den);
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    bool b_value;
    if (this_ptr->minimize(le, num, den, b_value)) {
      set_coefficient(env, j_num, build_java_coeff(env, num));
      set_coefficient(env, j_den, build_java_coeff(env, den));
      jobject j_boolean = bool_to_j_boolean_class(env, b_value);
      set_by_reference(env, j_ref_boolean, j_boolean);
      return true;
    }
    return false;
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_maximize__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_By_1Reference_2Lparma_1polyhedra_1library_Generator_2
(JNIEnv* env, jobject j_this, jobject j_le,
 jobject j_num, jobject j_den, jobject j_ref_boolean, jobject j_g) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    PPL_DIRTY_TEMP_COEFFICIENT(num);
    PPL_DIRTY_TEMP_COEFFICIENT(den);
    num = build_cxx_coeff(env, j_num);
    den = build_cxx_coeff(env, j_den);
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    bool b_value;
    Generator g = point();
    if (this_ptr->maximize(le, num, den, b_value, g)) {
      set_coefficient(env, j_num, build_java_coeff(env, num));
      set_coefficient(env, j_den, build_java_coeff(env, den));
      jobject j_boolean = bool_to_j_boolean_class(env, b_value);
      set_by_reference(env, j_ref_boolean, j_boolean);
      jobject j_g_result = build_java_generator(env, g);
      set_generator(env, j_g, j_g_result);
      return true;
    }
    return false;
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_minimize__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_By_1Reference_2Lparma_1polyhedra_1library_Generator_2
(JNIEnv* env, jobject j_this, jobject j_le,
 jobject j_num, jobject j_den, jobject j_ref_boolean, jobject j_g) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    PPL_DIRTY_TEMP_COEFFICIENT(num);
    PPL_DIRTY_TEMP_COEFFICIENT(den);
    num = build_cxx_coeff(env, j_num);
    den = build_cxx_coeff(env, j_den);
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    bool b_value;
    Generator g = point();
    if (this_ptr->minimize(le, num, den, b_value, g)) {
      set_coefficient(env, j_num, build_java_coeff(env, num));
      set_coefficient(env, j_den, build_java_coeff(env, den));
      jobject j_boolean = bool_to_j_boolean_class(env, b_value);
      set_by_reference(env, j_ref_boolean, j_boolean);
      jobject j_g_result = build_java_generator(env, g);
      set_generator(env, j_g, j_g_result);
      return true;
    }
    return false;
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_1frequency__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2
(JNIEnv* env, jobject j_this, jobject j_le,
 jobject j_freqn, jobject j_freqd, jobject j_valn, jobject j_vald) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    PPL_DIRTY_TEMP_COEFFICIENT(freqn);
    PPL_DIRTY_TEMP_COEFFICIENT(freqd);
    PPL_DIRTY_TEMP_COEFFICIENT(valn);
    PPL_DIRTY_TEMP_COEFFICIENT(vald);
    freqn = build_cxx_coeff(env, j_freqn);
    freqd = build_cxx_coeff(env, j_freqd);
    valn = build_cxx_coeff(env, j_valn);
    vald = build_cxx_coeff(env, j_vald);
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    if (this_ptr->frequency(le, freqn, freqd, valn, vald)) {
      set_coefficient(env, j_freqn, build_java_coeff(env, freqn));
      set_coefficient(env, j_freqd, build_java_coeff(env, freqd));
      set_coefficient(env, j_valn, build_java_coeff(env, valn));
      set_coefficient(env, j_vald, build_java_coeff(env, vald));
      return true;
    }
    return false;
  }
  CATCH_ALL;
  return false;
}





JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_contains
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Grid* y_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_y));
    return this_ptr->contains(*y_ptr);
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_strictly_1contains
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Grid* y_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_y));
    return this_ptr->strictly_contains(*y_ptr);
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_is_1disjoint_1from
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Grid* y_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_y));
    return this_ptr->is_disjoint_from(*y_ptr);
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_equals
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Grid* y_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_y));
    return *this_ptr == *y_ptr;
  }
  CATCH_ALL;
  return false;
}





JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_OK
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->OK();
  }
  CATCH_ALL;
  return false;
}





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_add_1constraint
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Constraint c = build_cxx_constraint(env, j_c);
  this_ptr->add_constraint(c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_add_1congruence
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Congruence c = build_cxx_congruence(env, j_c);
  this_ptr->add_congruence(c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_add_1grid_1generator
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid_Generator c = build_cxx_grid_generator(env, j_c);
  this_ptr->add_grid_generator(c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_add_1constraints
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Constraint_System cs = build_cxx_constraint_system(env, j_cs);
  this_ptr->add_constraints(cs);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_add_1congruences
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Congruence_System cs = build_cxx_congruence_system(env, j_cs);
  this_ptr->add_congruences(cs);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_add_1grid_1generators
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid_Generator_System cs = build_cxx_grid_generator_system(env, j_cs);
  this_ptr->add_grid_generators(cs);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_refine_1with_1constraint
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Constraint c = build_cxx_constraint(env, j_c);
  this_ptr->refine_with_constraint(c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_refine_1with_1congruence
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Congruence c = build_cxx_congruence(env, j_c);
  this_ptr->refine_with_congruence(c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_refine_1with_1constraints
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Constraint_System cs
    = build_cxx_constraint_system(env, j_cs);
  this_ptr->refine_with_constraints(cs);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_refine_1with_1congruences
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Congruence_System cs
    = build_cxx_congruence_system(env, j_cs);
  this_ptr->refine_with_congruences(cs);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_intersection_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  this_ptr->intersection_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_upper_1bound_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  this_ptr->upper_bound_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_difference_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  this_ptr->difference_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_concatenate_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  this_ptr->concatenate_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_time_1elapse_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  this_ptr->time_elapse_assign(*y_ptr);
}
CATCH_ALL






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_upper_1bound_1assign_1if_1exact
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Grid* y_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_y));
    return this_ptr->upper_bound_assign_if_exact(*y_ptr);
  }
  CATCH_ALL;
  return false;
}







JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_simplify_1using_1context_1assign
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Grid* y_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_y));
    return this_ptr->simplify_using_context_assign(*y_ptr);
  }
  CATCH_ALL;
  return false;
}





JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Grid_constrains
(JNIEnv* env, jobject j_this, jobject j_var) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    Variable v = build_cxx_variable(env, j_var);
    return this_ptr->constrains(v);
  }
  CATCH_ALL;
  return false;
}





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_unconstrain_1space_1dimension
(JNIEnv* env, jobject j_this, jobject j_var) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_var);
  this_ptr->unconstrain(v);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_unconstrain_1space_1dimensions
(JNIEnv* env, jobject j_this, jobject j_v_set) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variables_Set v_set = build_cxx_variables_set(env, j_v_set);
  this_ptr->unconstrain(v_set);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_affine_1image
(JNIEnv* env, jobject j_this, jobject j_v, jobject j_le, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->affine_image(v, le, c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_affine_1preimage
(JNIEnv* env, jobject j_this, jobject j_v, jobject j_le, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->affine_preimage(v, le, c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_bounded_1affine_1image
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_lhs, jobject j_rhs, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->bounded_affine_image(v, lhs, rhs, c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_bounded_1affine_1preimage
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_lhs, jobject j_rhs, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->bounded_affine_preimage(v, lhs, rhs, c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_generalized_1affine_1image__Lparma_1polyhedra_1library_Variable_2Lparma_1polyhedra_1library_Relation_1Symbol_2Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_relsym, jobject j_le, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->generalized_affine_image(v, relsym, le, c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_generalized_1affine_1preimage__Lparma_1polyhedra_1library_Variable_2Lparma_1polyhedra_1library_Relation_1Symbol_2Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_relsym, jobject j_le, jobject j_c) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->generalized_affine_preimage(v, relsym, le, c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_generalized_1affine_1image__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Relation_1Symbol_2Lparma_1polyhedra_1library_Linear_1Expression_2
(JNIEnv* env, jobject j_this,
 jobject j_lhs, jobject j_relsym, jobject j_rhs) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  this_ptr->generalized_affine_image(lhs, relsym, rhs);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_generalized_1affine_1preimage__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Relation_1Symbol_2Lparma_1polyhedra_1library_Linear_1Expression_2
(JNIEnv* env, jobject j_this,
 jobject j_lhs, jobject j_relsym, jobject j_rhs) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  this_ptr->generalized_affine_preimage(lhs, relsym, rhs);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_generalized_1affine_1image_1with_1congruence
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_relsym, jobject j_le, jobject j_c,
 jobject j_modulus) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  PPL_DIRTY_TEMP_COEFFICIENT(modulus);
  c = build_cxx_coeff(env, j_c);
  modulus = build_cxx_coeff(env, j_modulus);
  this_ptr->generalized_affine_image(v, relsym, le, c, modulus);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_generalized_1affine_1preimage_1with_1congruence
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_relsym, jobject j_le, jobject j_c,
 jobject j_modulus) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  PPL_DIRTY_TEMP_COEFFICIENT(modulus);
  c = build_cxx_coeff(env, j_c);
  modulus = build_cxx_coeff(env, j_modulus);
  this_ptr->generalized_affine_preimage(v, relsym, le, c, modulus);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_generalized_1affine_1image_1lhs_1rhs_1with_1congruence
(JNIEnv* env, jobject j_this,
 jobject j_lhs, jobject j_relsym, jobject j_rhs, jobject j_modulus) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  PPL_DIRTY_TEMP_COEFFICIENT(modulus);
  modulus = build_cxx_coeff(env, j_modulus);
  this_ptr->generalized_affine_image(lhs, relsym, rhs, modulus);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_generalized_1affine_1preimage_1lhs_1rhs_1with_1congruence
(JNIEnv* env, jobject j_this,
 jobject j_lhs, jobject j_relsym, jobject j_rhs, jobject j_modulus) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  PPL_DIRTY_TEMP_COEFFICIENT(modulus);
  modulus = build_cxx_coeff(env, j_modulus);
  this_ptr->generalized_affine_preimage(lhs, relsym, rhs, modulus);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_add_1space_1dimensions_1and_1embed
(JNIEnv* env, jobject j_this, jlong dim) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(dim);
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  this_ptr->add_space_dimensions_and_embed(ppl_dim);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_add_1space_1dimensions_1and_1project
(JNIEnv* env, jobject j_this, jlong dim) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(dim);
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  this_ptr->add_space_dimensions_and_project(ppl_dim);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_remove_1space_1dimensions
(JNIEnv* env, jobject j_this, jobject j_v_set) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variables_Set v_set = build_cxx_variables_set(env, j_v_set);
  this_ptr->remove_space_dimensions(v_set);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_remove_1higher_1space_1dimensions
(JNIEnv* env, jobject j_this, jlong dim) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(dim);
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  this_ptr->remove_higher_space_dimensions(ppl_dim);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_expand_1space_1dimension
(JNIEnv* env, jobject j_this, jobject j_variable, jlong dim) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(dim);
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_variable);
  this_ptr->expand_space_dimension(v, ppl_dim);
}
CATCH_ALL





JNIEXPORT void JNICALL Java_parma_1polyhedra_1library_Grid_fold_1space_1dimensions
(JNIEnv* env, jobject j_this, jobject j_v_set, jobject j_var) try {
  Variables_Set v_set = build_cxx_variables_set(env, j_v_set);
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_var);
  this_ptr->fold_space_dimensions(v_set, v);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_map_1space_1dimensions
(JNIEnv* env, jobject j_this, jobject j_p_func) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Partial_Function* p_func_ptr
    = reinterpret_cast<Partial_Function*>(get_ptr(env, j_p_func));
  this_ptr->map_space_dimensions(*p_func_ptr);
}
CATCH_ALL





JNIEXPORT void JNICALL Java_parma_1polyhedra_1library_Grid_drop_1some_1non_1integer_1points
(JNIEnv* env, jobject j_this, jobject j_complexity) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  switch (j_complexity_int) {
  case 0:
    this_ptr->drop_some_non_integer_points(POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr->drop_some_non_integer_points(SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr->drop_some_non_integer_points(ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
}
CATCH_ALL





JNIEXPORT void JNICALL Java_parma_1polyhedra_1library_Grid_drop_1some_1non_1integer_1points_12
(JNIEnv* env, jobject j_this, jobject j_v_set, jobject j_complexity) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variables_Set v_set = build_cxx_variables_set(env, j_v_set);
  jint j_complexity_int
    = env->CallIntMethod(j_complexity,
                         cached_FMIDs.Complexity_Class_ordinal_ID);
  CHECK_EXCEPTION_ASSERT(env);
  switch (j_complexity_int) {
  case 0:
    this_ptr->drop_some_non_integer_points(v_set, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr->drop_some_non_integer_points(v_set, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr->drop_some_non_integer_points(v_set, ANY_COMPLEXITY);
    break;
  default:
    assert(false);
    break;
  }
}
CATCH_ALL





JNIEXPORT jstring JNICALL
Java_parma_1polyhedra_1library_Grid_ascii_1dump
(JNIEnv* env, jobject j_this) {
  try {
    using namespace Parma_Polyhedra_Library::IO_Operators;
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    std::ostringstream s;
    this_ptr->ascii_dump(s);
    std::string str = s.str();
    return env->NewStringUTF(str.c_str());
  }
  CATCH_ALL;
  return 0;
}





JNIEXPORT jlong JNICALL
Java_parma_1polyhedra_1library_Grid_external_1memory_1in_1bytes
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->external_memory_in_bytes();
  }
  CATCH_ALL;
  return 0;
}

JNIEXPORT jlong JNICALL
Java_parma_1polyhedra_1library_Grid_total_1memory_1in_1bytes
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->total_memory_in_bytes();
  }
  CATCH_ALL;
  return 0;
}








JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_congruence_1widening_1assign
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_by_ref_int) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  if (j_by_ref_int == NULL)
    this_ptr->congruence_widening_assign(*y_ptr);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->congruence_widening_assign(*y_ptr, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_generator_1widening_1assign
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_by_ref_int) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  if (j_by_ref_int == NULL)
    this_ptr->generator_widening_assign(*y_ptr);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->generator_widening_assign(*y_ptr, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL








JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_widening_1assign
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_by_ref_int) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  if (j_by_ref_int == NULL)
    this_ptr->widening_assign(*y_ptr);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->widening_assign(*y_ptr, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL







JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_limited_1congruence_1extrapolation_1assign
(JNIEnv* env, jobject j_this, jobject j_y,
 jobject j_cs, jobject j_by_ref_int) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  Congruence_System cs = build_cxx_congruence_system(env, j_cs);
  if (j_by_ref_int == NULL)
    this_ptr->limited_congruence_extrapolation_assign(*y_ptr, cs);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->limited_congruence_extrapolation_assign
      (*y_ptr, cs, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_limited_1generator_1extrapolation_1assign
(JNIEnv* env, jobject j_this, jobject j_y,
 jobject j_cs, jobject j_by_ref_int) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  Congruence_System cs = build_cxx_congruence_system(env, j_cs);
  if (j_by_ref_int == NULL)
    this_ptr->limited_generator_extrapolation_assign(*y_ptr, cs);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->limited_generator_extrapolation_assign
      (*y_ptr, cs, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL












JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_wrap_1assign
(JNIEnv* env, jobject j_this, jobject j_vars,
 jobject j_w, jobject j_r, jobject j_o, jobject j_cs,
 jlong j_cx, jboolean j_wrap_individually) try {
  Grid* this_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_this));
  Variables_Set vars = build_cxx_variables_set(env, j_vars);
  Bounded_Integer_Type_Width w = build_cxx_bounded_width(env, j_w);
  Bounded_Integer_Type_Representation r = build_cxx_bounded_rep(env, j_r);
  Bounded_Integer_Type_Overflow o = build_cxx_bounded_overflow(env, j_o);
  Constraint_System cs = build_cxx_constraint_system(env, j_cs);
  unsigned cx = jtype_to_unsigned<unsigned int>(j_cx);
  this_ptr->wrap_assign(vars, w, r, o, &cs, cx, j_wrap_individually);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Grid_free
(JNIEnv* env, jobject j_this) try {
  if (!is_java_marked(env, j_this)) {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    delete this_ptr;
    void* null_ptr = 0;
    set_ptr(env, j_this, null_ptr);
  }
}
CATCH_ALL






JNIEXPORT jint JNICALL
Java_parma_1polyhedra_1library_Grid_hashCode
(JNIEnv* env, jobject j_this) {
  try {
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    return this_ptr->hash_code();
  }
  CATCH_ALL;
  return 0;
}





JNIEXPORT jstring JNICALL
Java_parma_1polyhedra_1library_Grid_toString
(JNIEnv* env, jobject j_this) {
  try {
    using namespace Parma_Polyhedra_Library::IO_Operators;
    Grid* this_ptr
      = reinterpret_cast<Grid*>(get_ptr(env, j_this));
    std::ostringstream s;
    s << *this_ptr;
    std::string str = s.str();
    return env->NewStringUTF(str.c_str());
  }
  CATCH_ALL;
  return 0;
}





